import { gql, useQuery } from "@apollo/client";
import { Link, useRouter } from "@curi/react-dom";
import {
  List,
  ListItem,
  Typography,
  ListItemText,
  ListItemSecondaryAction,
  IconButton,
  Menu,
  MenuItem,
  PopoverPosition,
} from "@mui/material";
import { FC, MouseEventHandler, useMemo, useState } from "react";
import { Pipeline, Project } from "../generated/graphql";
import { CallMerge, Edit } from "@mui/icons-material";
import { any, values } from "ramda";

interface Props {
  projectId: string;
}

const PIPELINES = gql`
  query Pipelines($projectId: String!) {
    pipelines(projectId: $projectId) {
      id
      name
      branch
    }
    project(id: $projectId) {
      id
    }
  }
`;

export const PipelineList: FC<Props> = ({ projectId }) => {
  const { data } = useQuery<
    { pipelines: Pipeline[]; project: Project },
    { projectId: string }
  >(PIPELINES, {
    variables: { projectId },
  });
  const pipelines = useMemo(() => {
    return data?.pipelines?.map((pipeline) => ({
      ...pipeline,
      project: data?.project,
    }));
  }, [data]);

  const [contextMenu, setContextMenu] = useState<PopoverPosition>();

  const handleContextMenu: MouseEventHandler = (event) => {
    event.preventDefault();
    setContextMenu({ top: event.clientY - 4, left: event.clientX - 2 });
  };

  const handleClose = () => {
    setContextMenu(undefined);
  };

  return (
    <>
      <List>
        {pipelines?.map((pipeline) => (
          <Link
            name="pipeline-commits"
            params={{ pipelineId: pipeline.id, projectId: projectId }}
            key={pipeline.id}
            onContextMenu={handleContextMenu}
          >
            <Item pipeline={pipeline} />
          </Link>
        ))}
      </List>
      <Menu
        keepMounted
        open={!!contextMenu}
        onClose={handleClose}
        anchorReference="anchorPosition"
        anchorPosition={
          contextMenu && any(Boolean, values(contextMenu))
            ? contextMenu
            : undefined
        }
      >
        <MenuItem onClick={handleClose}>Copy</MenuItem>
        <MenuItem onClick={handleClose}>Print</MenuItem>
        <MenuItem onClick={handleClose}>Highlight</MenuItem>
        <MenuItem onClick={handleClose}>Email</MenuItem>
      </Menu>
    </>
  );
};

const Item = ({ pipeline }: { pipeline: Pipeline }) => {
  const { navigate, url } = useRouter();
  const modify: MouseEventHandler = (ev) => {
    ev.preventDefault();
    navigate({
      url: url({
        name: "edit-pipeline",
        params: { pipelineId: pipeline.id, projectId: pipeline.project.id },
      }),
    });
  };
  return (
    <ListItem button>
      <ListItemText
        primary={pipeline.name}
        secondary={
          <Typography component="span" variant="body2" color="textSecondary">
            <CallMerge fontSize="small" />
            {pipeline.branch}
          </Typography>
        }
      />
      <ListItemSecondaryAction>
        <IconButton edge="end" aria-label="edit" onClick={modify} size="large">
          <Edit />
        </IconButton>
      </ListItemSecondaryAction>
    </ListItem>
  );
};
